
from time import sleep
from elasticsearch import ElasticsearchException
from queue import Full
# Module from the same package
from es_translator.logger import logger


class FatalTranslationException(Exception):
    pass


def translation_worker(queue, shared_fatal_error):
    """
    Worker function that translates documents from the queue in parallel.

    Args:
        queue: JoinableQueue for retrieving documents to be translated.
        shared_fatal_error: Shared variable to track fatal errors.

    Notes:
        This function continuously retrieves documents from the queue and translates them until a fatal error occurs
        or the queue is empty. It handles different types of exceptions and updates the shared_fatal_error if needed.
    """
    while not shared_fatal_error.value:
        try:
            es_translator, hit = queue.get(True)
            es_translator.translate_document(hit)
            sleep(es_translator.throttle / 1000)
        except ElasticsearchException as error:
            handle_elasticsearch_exception(error, hit)
            shared_fatal_error.value = error
        except Full:
            handle_timeout_reached(es_translator.pool_timeout)
        except Exception as error:
            handle_exception(error, hit)
        finally:
            queue.task_done()
    queue.close()


def handle_elasticsearch_exception(error, index):
    # Handle Elasticsearch exception
    logger.error('An error occurred when saving doc %s' % index.meta.id)
    logger.error(error)


def handle_timeout_reached(pool_timeout):
    # Handle timeout reached
    logger.warning('Timeout reached (%ss).' % pool_timeout)


def handle_exception(error, hit):
    # Handle other exceptions
    logger.warning('Unable to translate doc %s' % hit.meta.id)
    logger.warning(error)



import os
from typing import Any, Dict
from celery import Celery

REDIS_URL = os.environ.get('REDIS_URL', 'redis://localhost:6379/0')

app = Celery('EsTranslator', broker=REDIS_URL)
app.conf.task_default_queue = 'es_translator:default'


@app.task
def translate_document_task(
        translator_options: Dict[str, Any], document_params: Dict[str, Any]) -> None:
    """Celery task to translate a document.

    Args:
        translator_options (Dict[str, Any]): A dictionary of options for the translator.
        document_params (Dict[str, Any]): A dictionary of parameters to find the document.
    """
    from es_translator import EsTranslator
    # Initialize an EsTranslator with the options from the task
    es_translator = EsTranslator(translator_options)
    # Find the document using params
    document = es_translator.find_document(document_params)
    # Translate the document
    es_translator.translate_document(document)



import tempfile

from argostranslate import package as argospackage
from argostranslate import translate as argostranslate
from pathlib import Path
from filelock import FileLock, Timeout

from ..abstract import AbstractInterpreter
from ...logger import logger


class ArgosPairNotAvailable(Exception):
    """
    Exception raised when the necessary language pair is not available for download.
    """


class ArgosPackageDownloadLockTimeout(Exception):
    """
    Exception raised when the necessary language cannot be downloaded after a lock reach its timeout.
    """


class Argos(AbstractInterpreter):
    """
    Argos is a class that extends the AbstractInterpreter.

    This class is responsible for handling translation tasks, specifically using the Argos interpreter.

    Attributes:
        name (str): The name of the interpreter.
    """
    name = 'ARGOS'

    def __init__(
            self,
            source=None,
            target=None,
            intermediary=None,
            pack_dir=None):
        """
        Initializes the Argos interpreter with source and target language codes.

        The function also checks if the necessary language pair is available, and downloads it if necessary.

        Args:
            source (str): The source language code.
            target (str): The target language code.
            intermediary (str, optional): The intermediary language code. Defaults to None.
            pack_dir (str, optional): The directory of language packs. This option will be ignored. Defaults to None.

        Raises:
            Exception: If the necessary language pair is not available and cannot be downloaded.
        """
        super().__init__(source, target)
        # Raise an exception if an intermediary language is provded
        if intermediary is not None:
            logger.warn(
                'Argos interpreter doesnt support intermediary language')
        if pack_dir is not None:
            logger.warn(
                'Argos interpreter doesnt support custom pack directory')
        # Raise an exception if the language pair is unkown
        if not self.is_pair_available and self.has_pair:
            try:
                self.download_necessary_languages()
            except ArgosPairNotAvailable:
                raise Exception('The pair %s is not available' % self.pair)
        else:
            logger.info('Existing package(s) found for pair %s' % self.pair)

    @property
    def is_pair_available(self):
        """
        Checks if the necessary language pair is available in the installed packages.

        Returns:
            bool: True if the language pair is available, False otherwise.
        """
        for package in argospackage.get_installed_packages():
            if package.from_code == self.source_alpha_2 and package.to_code == self.target_alpha_2:
                return True
        return False

    @property
    def local_languages(self):
        """
        Gets the codes for the installed languages.

        This method retrieves the installed languages and returns their codes as a list.
        If an AttributeError is encountered, it returns an empty list.

        Returns:
            list of str: The codes for the installed languages.
        """
        try:
            installed_languages = argostranslate.get_installed_languages()
            return [lang.code for lang in installed_languages]
        except AttributeError:
            return []

    @property
    def local_languages(self):
        """
        Gets the codes for the installed languages.

        This method retrieves the installed languages and returns their codes as a list.
        If an AttributeError is encountered, it returns an empty list.

        Returns:
            list of str: The codes for the installed languages.
        """
        try:
            installed_languages = argostranslate.get_installed_languages()
            return [lang.code for lang in installed_languages]
        except AttributeError:
            return []

    def update_package_index(self):
        """
        Updates the package index.
        """
        argospackage.update_package_index()

    def find_necessary_package(self):
        """
        Finds the necessary language package.

        This method loops over the available packages to find the necessary package based on the source and target language codes.
        If the package cannot be found, it raises an ArgosPairNotAvailable exception.

        Returns:
            Package: The necessary language package.

        Raises:
            ArgosPairNotAvailable: If the necessary language package could not be found.
        """
        for package in argospackage.get_available_packages():
            if package.from_code == self.source_alpha_2 and package.to_code == self.target_alpha_2:
                return package
        raise ArgosPairNotAvailable

    def is_package_installed(self, package):
        """
        Checks if a package is installed.

        Args:
            package (Package): The package to check.

        Returns:
            bool: True if the package is installed, False otherwise.
        """
        return package in argospackage.get_installed_packages()

    def download_and_install_package(self, package):
        """
        Downloads and installs a language package.

        This method locks the download using a file lock based on the package's source and target language codes.
        If the package is not installed, it is downloaded and installed.

        Args:
            package (Package): The package to download and install.

        Raises:
            Timeout: If a lock on the download path cannot be acquired within the timeout duration.
        """
        try:
            temp_dir = Path(tempfile.gettempdir())
            lock_path = temp_dir / f'{package.from_code}_{package.to_code}.lock'
            
            with FileLock(lock_path, timeout=600).acquire(timeout=600):
                if self.is_package_installed(package):
                    return
                download_path = package.download()
                logger.info(f'Installing Argos package {package}')
                return argospackage.install_from_path(download_path)
        except Timeout:
            raise ArgosPackageDownloadLockTimeout(
                f'Another instance of the program is downloading the package {package}. Please try again later.')

    def download_necessary_languages(self):
        """
        Downloads necessary language packages if they are not installed already.

        This method performs the following steps:
        1. Updates the package index.
        2. Finds the necessary package.
        3. Downloads and installs the package.

        Raises:
            ArgosPairNotAvailable: If the necessary language package could not be found.
            Timeout: If a lock on the download path cannot be acquired within the timeout duration.
        """
        self.update_package_index()
        necessary_package = self.find_necessary_package()
        self.download_and_install_package(necessary_package)

    @property
    def translation(self):
        """
        Returns a Translation object for the source and target languages.

        Raises:
            Exception: If either the source or target language is not installed.

        Returns:
            Translation: The Translation object for the source and target languages.
        """
        installed_languages = argostranslate.get_installed_languages()
        source = list(
            filter(
                lambda x: x.code == self.source_alpha_2,
                installed_languages))[0]
        target = list(
            filter(
                lambda x: x.code == self.target_alpha_2,
                installed_languages))[0]
        return source.get_translation(target)

    def translate(self, input):
        """
        Translates the input text from the source language to the target language.

        Args:
            input (str): The input text in the source language.

        Returns:
            str: The translated text in the target language.
        """
        return self.translation.translate(input)



import logging
import sys
from syslog import LOG_LOCAL7

import coloredlogs
from logging.handlers import SysLogHandler

logger = logging.getLogger('es-translator')
logger.setLevel(logging.INFO)


def default_log_formatter() -> logging.Formatter:
    return logging.Formatter(
        '%(asctime)s :: %(name)s :: %(levelname)s :: %(message)s')


def add_syslog_handler(
        address: str = 'localhost',
        port: int = 514,
        facility: int = LOG_LOCAL7) -> None:
    sysLogFormatter = default_log_formatter()
    sysLogHandler = SysLogHandler(address=(address, port), facility=facility)
    sysLogHandler.setLevel(logging.INFO)
    sysLogHandler.setFormatter(sysLogFormatter)
    logger.addHandler(sysLogHandler)


def add_stdout_handler(level: int = logging.ERROR) -> None:
    fmt = '%(levelname)s %(message)s'
    logger.addHandler(logging.StreamHandler(sys.stdout))
    coloredlogs.install(level=level, logger=logger, fmt=fmt)



import sys
from coloredlogs import StandardErrorHandler
from contextlib import contextmanager
from elasticsearch import Elasticsearch, ElasticsearchException
from elasticsearch_dsl import Document, Search
from multiprocessing import Pool, JoinableQueue, Manager
from queue import Full
from os import path
from rich.progress import Progress
from typing import Any, ContextManager, Dict, List

# Module from the same package
from es_translator.interpreters import Apertium, Argos
from es_translator.logger import logger
from es_translator.worker import translation_worker, FatalTranslationException
from es_translator.tasks import translate_document_task
from es_translator.es import TranslatedHit


class EsTranslator:
    def __init__(self, options: Dict[str, Any]):
        """Initialize the Elasticsearch translator.

        Args:
            options (Dict[str, Any]): A dictionary of options needed to set up the translator.
        """
        self.url = options['url']
        self.index = options['index']
        self.source_language = options['source_language']
        self.target_language = options['target_language']
        self.intermediary_language = options['intermediary_language']
        self.source_field = options['source_field']
        self.target_field = options['target_field']
        self.query_string = options['query_string']
        self.data_dir = options['data_dir']
        self.scan_scroll = options['scan_scroll']
        self.dry_run = options.get('dry_run', False)
        self.pool_size = options['pool_size']
        self.pool_timeout = options['pool_timeout']
        self.throttle = options['throttle']
        self.progressbar = options.get('progressbar', False)
        self.interpreter_name = options['interpreter']
        self.plan = options.get('plan', False)

    @property
    def no_progressbar(self) -> bool:
        """Check if the progressbar option is set to False.

        Returns:
            bool: True if the progressbar option is False, else False.
        """
        return not self.progressbar

    def start(self) -> None:
        """Starts or plans the translation process."""
        if self.plan:
            self.start_later()
        else:
            self.start_now()

    def start_now(self) -> None:
        """Starts the translation process."""
        self.instantiate_interpreter()
        total = self.search().count()
        desc = 'Translating %s document(s)' % total
        with self.print_done(desc):
            search = self.configure_search()
            translation_queue = self.create_translation_queue()
            with self.with_shared_fatal_error() as shared_fatal_error:
                self.translate_documents_in_pool(
                    search, translation_queue, shared_fatal_error, total)

    def start_later(self):
        """Plan the translation process."""
        self.instantiate_interpreter()
        total = self.search().count()
        desc = f"Planning translation for {total} document{'s'[:total^1]}"
        with self.print_done(desc):
            search = self.configure_search()
            for hit in search.scan():
                logger.info(f'Planned translation for doc {hit.meta.id}')
                translate_document_task.delay(self.options, hit.meta.to_dict())

    @property
    def options(self):
        return {
            'url': self.url,
            'index': self.index,
            'source_language': self.source_language,
            'target_language': self.target_language,
            'intermediary_language': self.intermediary_language,
            'source_field': self.source_field,
            'target_field': self.target_field,
            'query_string': self.query_string,
            'data_dir': self.data_dir,
            'scan_scroll': self.scan_scroll,
            'dry_run': self.dry_run,
            'pool_size': self.pool_size,
            'pool_timeout': self.pool_timeout,
            'throttle': self.throttle,
            'progressbar': self.progressbar,
            'interpreter': self.interpreter_name
        }

    def instantiate_interpreter(self) -> Any:
        """Instantiates the interpreter.

        Returns:
            Any: An instance of the interpreter.
        """
        if not hasattr(self, 'interpreter'):
            with self.print_done(f'Instantiating {self.interpreter_name} interpreter'):
                self.interpreter = self.init_interpreter()
        return self.interpreter

    def configure_search(self) -> Search:
        """Configures the search object.

        Returns:
            Search: A configured search object.
        """
        search = self.search()
        search = search.source(self.search_source)
        search = search.params(scroll=self.scan_scroll, size=self.pool_size)
        return search
    
    @property
    def search_source(self) -> List[str]:
        """Gets the list of fields to use in the search.

        Returns: 
            List[str]: list of fields to use in the search.
        """
        if self.plan:
            return ['_routing', '_id']
        return [self.source_field, self.target_field, '_routing', '_id']


    def create_translation_queue(self) -> JoinableQueue:
        """Creates a queue that can translate documents in parallel.

        Returns:
            JoinableQueue: A queue for parallel document translation.
        """
        return JoinableQueue(self.pool_size)

    @contextmanager
    def with_shared_fatal_error(self) -> ContextManager:
        """Creates a context manager for managing shared fatal errors.

        Returns:
            ContextManager: A context manager.
        """
        with Manager() as manager:
            yield manager.Value('b', None)

    def find_document(self, params: Dict[str, str]):
        using = Elasticsearch(self.url)
        routing = getattr(params, 'routing', params['id'])
        return Document.get(
            index=params['index'],
            id=params['id'],
            routing=routing,
            using=using)

    def translate_document(self, hit):
        self.instantiate_interpreter()
        # Translate the document
        logger.info(f'Translating doc {hit.meta.id}')
        translated_hit = TranslatedHit(
            hit, self.source_field, self.target_field)
        translated_hit.add_translation(self.interpreter)
        logger.info(f'Translated doc {hit.meta.id}')
        # Save the translated document if not in dry run mode
        if not self.dry_run:
            client = Elasticsearch(self.url)
            translated_hit.save(client)
            logger.info(f'Saved translation for doc {hit.meta.id}')

    def translate_documents_in_pool(
            self,
            search: Search,
            translation_queue: JoinableQueue,
            shared_fatal_error: Manager,
            total: int) -> None:
        """Translates documents.

        Args:
            search (Search): A search object.
            translation_queue (JoinableQueue): A queue for parallel document translation.
            shared_fatal_error (Manager): A shared manager for fatal errors.
            total (int): The total number of documents.
        """
        with Pool(self.pool_size, translation_worker, (translation_queue, shared_fatal_error)):
            with Progress(disable=self.no_progressbar, transient=True) as progress:
                task = progress.add_task(
                    f"Translating {total} document{'s'[:total^1]}", total=total)
                for hit in search.scan():
                    self.process_document(
                        translation_queue, hit, progress, task, shared_fatal_error)
                translation_queue.join()

    def process_document(
            self,
            translation_queue: JoinableQueue,
            hit: Any,
            progress: Progress,
            task: Any,
            shared_fatal_error: Manager) -> None:
        """Processes a document.

        Args:
            translation_queue (JoinableQueue): A queue for parallel document translation.
            hit (Any): The document to be translated.
            index (int): The index of the document.
            progress (Progress): A progress object.
            task (Any): The current task.
            shared_fatal_error (Manager): A shared manager for fatal errors.
        """
        translation_queue.put((self, hit), True, self.pool_timeout)
        progress.advance(task)
        if shared_fatal_error.value:
            raise FatalTranslationException(shared_fatal_error.value)

    def search(self) -> Search:
        """Executes a search query.

        Returns:
            Search: The search result.
        """
        using = Elasticsearch(self.url)
        search = Search(index=self.index, using=using)
        if self.query_string:
            search = search.query("query_string", query=self.query_string)
        return search

    def init_interpreter(self) -> Any:
        """Initializes the interpreter.

        Returns:
            Any: The initialized interpreter.
        """
        pack_dir = path.join(self.data_dir, 'packs', self.interpreter_name)
        interpreters = (Apertium, Argos,)
        Interpreter = next(
            i for i in interpreters if i.name.lower() == self.interpreter_name.lower())
        return Interpreter(
            self.source_language,
            self.target_language,
            self.intermediary_language,
            pack_dir)

    def print_flush(self, string: str) -> None:
        """Prints and flushes a string to stdout.

        Args:
            string (str): The string to be printed.
        """
        sys.stdout.write('\r{0}'.format(string))
        sys.stdout.flush()

    @property
    def stdout_loglevel(self) -> int:
        """Gets the log level of stdout.

        Returns:
            int: The log level of stdout.
        """
        try:
            handler = next(
                h for h in logger.handlers if isinstance(
                    h, StandardErrorHandler))
            return getattr(handler, 'level', 0)
        except StopIteration:
            return 0

    @contextmanager
    def print_done(self, string: str) -> ContextManager:
        """Prints a string and yields.

        Args:
            string (str): The string to be printed.

        Returns:
            ContextManager: A context manager.
        """
        logger.info(string)
        if self.stdout_loglevel > 20:
            string = '\r%s...' % string
            self.print_flush(string)
            try:
                yield
                print('{0} \033[92mdone\033[0m'.format(string))
            except (FatalTranslationException, ElasticsearchException, Full) as error:
                logger.error(error, exc_info=True)
                print('{0} \033[91merror\033[0m'.format(string))
                sys.exit(1)
        else:
            yield



from deb_pkg_tools.control import parse_deb822
from fileinput import FileInput
from functools import lru_cache
from glob import glob
from os.path import basename, join, isfile, dirname, abspath
from sh import dpkg_deb, mkdir, pushd, cp, rm
from urllib import request
# Module from the same package
from es_translator.alpha import to_alpha_2, to_alpha_3, to_alpha_3_pair
from es_translator.logger import logger
from es_translator.symlink import create_symlink

REPOSITORY_URL = "https://apertium.projectjj.com/apt/nightly"
PACKAGES_FILE_URL = "%s/dists/focal/main/binary-amd64/Packages" % REPOSITORY_URL


class ApertiumRepository:
    def __init__(self, cache_dir=None):
        # Create a temporary pack dir (if needed)
        self.cache_dir = abspath(cache_dir)

    @property
    @lru_cache()
    def control_file_content(self):
        response = request.urlopen(PACKAGES_FILE_URL)
        data = response.read()
        return data.decode('utf-8')

    @property
    @lru_cache()
    def packages(self):
        def isnt_empty(c): return c is not None and c != ''
        control_strings = self.control_file_content.split('\n\n')
        control_strings = list(filter(isnt_empty, control_strings))
        return list(map(parse_deb822, control_strings))

    @property
    @lru_cache()
    def pair_packages(self):
        return list(filter(self.is_apertium_pair, self.packages))

    def find_package(self, package):
        def is_package(c): return c.get(
            'Package') == package or c.get('Provides') == package
        try:
            return next(filter(is_package, self.packages))
        except StopIteration:
            logger.warning('Unable to found package %s' % package)
            return None

    def find_pair_package(self, pair):
        pair = to_alpha_3_pair(pair)
        pair_inversed = '-'.join(pair.split('-')[::-1])

        def is_pair(c):
            return c.get('Package', '').endswith(pair) \
                or c.get('Package', '').endswith(pair_inversed)
        try:
            return next(filter(is_pair, self.pair_packages))
        except StopIteration:
            return None

    def is_apertium_pair(self, control):
        try:
            parts = control['Package'].split('-')
            return len(parts) == 3 and parts[0] == 'apertium'
        except KeyError:
            return False

    def download_package(self, name, force=False):
        package = self.find_package(name)
        package_url = REPOSITORY_URL + '/' + package['Filename']
        package_dir = join(self.cache_dir, name)
        package_file = join(package_dir, 'package') + '.deb'
        mkdir('-p', package_dir)
        # Don't download the file twice
        if force or not isfile(package_file):
            logger.info('Downloading package %s' % name)
            request.urlretrieve(package_url, package_file)
        return package_file

    def download_pair_package(self, pair):
        pair_package = self.find_pair_package(pair)
        if pair_package is not None:
            return self.download_package(pair_package.get('Package'))
        else:
            raise Exception('No pair package  available for "%s"' % pair)

    def replace_in_file(self, file, target, replacement):
        with FileInput(file, inplace=True) as fileinput:
            for line in fileinput:
                print(line.replace(target, replacement), end='')

    def extract_pair_package(self, file, extraction_dir='.'):
        workdir = dirname(file)
        with pushd(workdir):
            # Extract the file from the .deb
            dpkg_deb('-x', file, extraction_dir)
            # Copy the files we need
            cp('-rlf', glob('usr/share/apertium/*'), extraction_dir)
            # Remove everything else
            rm('-Rf', 'usr')
            # Rewrite paths in modes files
            for mode in glob('modes/*.mode'):
                self.replace_in_file(mode, '/usr/share/apertium', workdir)
        return workdir

    def create_pair_package_alias(self, package_dir):
        extraction_dir = dirname(package_dir) + '/'
        [source, target] = basename(package_dir).split(
            'apertium-')[-1].split('-')
        if len(source) == 2:
            aliases = (to_alpha_3(source), to_alpha_3(target))
        else:
            aliases = (to_alpha_2(source), to_alpha_2(target))
        # Build the alias dir using the alias
        alias_dir = join(extraction_dir, 'apertium-%s-%s' % aliases)
        mode_file = join(
            extraction_dir, 'modes', '%s-%s.mode' %
            (source, target))
        mode_alias_file = join(extraction_dir, 'modes', '%s-%s.mode' % aliases)
        # Use a symbolic links
        create_symlink(package_dir, alias_dir)
        create_symlink(mode_file, mode_alias_file)
        return alias_dir

    def install_pair_package(self, pair):
        logger.info('Installing pair package %s' % pair)
        package_file = self.download_pair_package(pair)
        package_dir = self.extract_pair_package(package_file)
        alias_dir = self.create_pair_package_alias(package_dir)
        self.import_modes(clear=False)
        return package_dir

    def clear_modes(self):
        with pushd(self.cache_dir):
            rm('-Rf', 'modes')

    def import_modes(self, clear=True):
        with pushd(self.cache_dir):
            mkdir('-p', 'modes')
            # Copy all the mode files
            cp(glob('./*/modes/*.mode'), './modes')



from abc import ABC, abstractmethod
from os.path import abspath

from ..alpha import to_alpha_2, to_alpha_3, to_name


class AbstractInterpreter(ABC):
    name = 'ABSTRACT'

    def __init__(
            self,
            source=None,
            target=None,
            intermediary=None,
            pack_dir=None):
        self.source = source
        self.target = target
        self.intermediary = intermediary
        # Create a temporary pack dir (if needed) to download language packs
        if pack_dir is not None:
            self.pack_dir = abspath(pack_dir)

    @property
    def name(self):
        return self.__class__.name

    @property
    def source_alpha_2(self):
        return to_alpha_2(self.source)

    @property
    def source_alpha_3(self):
        return to_alpha_3(self.source)

    @property
    def source_name(self):
        return to_name(self.source_alpha_2)

    @property
    def target_alpha_2(self):
        return to_alpha_2(self.target)

    @property
    def target_alpha_3(self):
        return to_alpha_3(self.target)

    @property
    def intermediary_alpha_3(self):
        return to_alpha_3(self.intermediary)

    @property
    def target_name(self):
        return to_name(self.target_alpha_2)

    @property
    def pair(self):
        return '%s-%s' % (self.source, self.target)

    @property
    def pair_alpha_3(self):
        return '%s-%s' % (self.source_alpha_3, self.target_alpha_3)

    @property
    def pair_inverse(self):
        return '%s-%s' % (self.target, self.source)

    @property
    def has_pair(self):
        return self.source is not None and self.target is not None

    @abstractmethod
    def translate(self, input):
        return input



from pycountry import languages


def to_alpha_2(code):
    if len(code) == 3:
        return languages.get(alpha_3=code).alpha_2
    else:
        return code


def to_alpha_3(code):
    if len(code) == 2:
        return languages.get(alpha_2=code).alpha_3
    else:
        return code


def to_name(alpha_2):
    return languages.get(alpha_2=alpha_2).name


def to_alpha_3_pair(pair):
    [source, target] = pair.split('-')
    return '%s-%s' % (to_alpha_3(source), to_alpha_3(target))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['es_translator',
 'es_translator.interpreters',
 'es_translator.interpreters.apertium',
 'es_translator.interpreters.argos']

package_data = \
{'': ['*']}

install_requires = \
['argostranslate>=1.7,<2.0',
 'celery[redis]>=5.3.1,<6.0.0',
 'click>=8,<9',
 'coloredlogs',
 'deb-pkg-tools>=8.4,<9.0',
 'elasticsearch-dsl>=7.4.0,<8.0.0',
 'elasticsearch>=7.10,<7.11',
 'filelock>=3.12.2,<4.0.0',
 'pycountry>=22.3,<23.0',
 'rich>=12,<13',
 'sh>=1,<2',
 'torch>=1.13,<2.0',
 'urllib3>=1.26,<2.0']

entry_points = \
{'console_scripts': ['es-translator = es_translator.cli:translate',
                     'es-translator-pairs = es_translator.cli:pairs',
                     'es-translator-tasks = es_translator.cli:tasks']}

setup_kwargs = {
    'name': 'es-translator',
    'version': '1.7.3',
    'description': 'A lazy yet bulletproof machine translation tool for Elastichsearch.',
    'long_description': "# ES Translator [![](https://img.shields.io/github/actions/workflow/status/icij/es-translator/main.yml)](https://github.com/ICIJ/es-translator/actions) [![](https://img.shields.io/pypi/pyversions/es-translator)](https://pypi.org/project/es-translator/) \n\nA lazy yet bulletproof machine translation tool for Elastichsearch.\n\n## Installation (Ubuntu)\n\nInstall Apertium:\n\n```bash\nwget https://apertium.projectjj.com/apt/install-nightly.sh -O - | sudo bash\nsudo apt install apertium-all-dev\n```\n\nThen install es-translator with pip:\n\n```bash\npython3 -m pip install --user es-translator\n```\n\n## Installation (Docker)\n\nNothing to do as long as you have Docker on your system:\n\n```\ndocker run -it icij/es-translator es-translator --help\n```\n\n## Usage\n\nThe primarly command from EsTranslator to translate documents is `es-translator`:\n\n\n```\nUsage: es-translator [OPTIONS]\n\nOptions:\n  -u, --url TEXT                  Elastichsearch URL\n  -i, --index TEXT                Elastichsearch Index  [required]\n  -r, --interpreter TEXT          Interpreter to use to perform the\n                                  translation\n  -s, --source-language TEXT      Source language to translate from\n                                  [required]\n  -t, --target-language TEXT      Target language to translate to  [required]\n  --intermediary-language TEXT    An intermediary language to use when no\n                                  translation is available between the source\n                                  and the target. If none is provided this\n                                  will be calculated automatically.\n  --source-field TEXT             Document field to translate\n  --target-field TEXT             Document field where the translations are\n                                  stored\n  -q, --query-string TEXT         Search query string to filter result\n  -d, --data-dir PATH             Path to the directory where to language\n                                  model will be downloaded\n  --scan-scroll TEXT              Scroll duration (set to higher value if\n                                  you're processing a lot of documents)\n  --dry-run                       Don't save anything in Elasticsearch\n  --pool-size INTEGER             Number of parallel processes to start\n  --pool-timeout INTEGER          Timeout to add a translation\n  --throttle INTEGER              Throttle between each translation (in ms)\n  --syslog-address TEXT           Syslog address\n  --syslog-port INTEGER           Syslog port\n  --syslog-facility TEXT          Syslog facility\n  --stdout-loglevel TEXT          Change the default log level for stdout\n                                  error handler\n  --progressbar / --no-progressbar\n                                  Display a progressbar\n  --plan                          Plan translations into a queue instead of\n                                  processing them npw\n  --broker-url TEXT               Celery broker URL (only needed when planning\n                                  translation)\n  --help                          Show this message and exit.\n```\n\nLearn more about how to use this command in the [Usage Documentation](https://icij.github.io/es-translator/usage/).\n\n## API\n\nYou can explore the [API Documentation](https://icij.github.io/es-translator/api/) for more information.\n\n## Contributing\n\nContributions are welcome! If you encounter any issues or have suggestions for improvements, please open an issue on the [GitHub repository](https://github.com/icij/es-translator). If you're willing to help, check the page about [how to contribute](https://icij.github.io/es-translator/contributing/) to this project.\n\n## License\n\nThis project is licensed under the MIT License. See the [LICENSE](https://github.com/icij/es-translator/blob/main/LICENSE.md) file for more details.\n\n",
    'author': 'ICIJ',
    'author_email': 'engineering@icij.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.2,<3.11',
}


setup(**setup_kwargs)
